import java.awt.*;
import java.awt.event.*;
import java.io.Serializable;
import java.util.Date;
import java.util.Calendar;
import java.text.*;
import java.lang.reflect.*;
import java.beans.*;

public class CalendarWidget extends Panel implements Serializable, 
  ActionListener{
    TextField text;    
    Button change;
    DateFormat format;
    CalendarFrame calendar;
    private Date date = new Date();
    private PropertyChangeSupport changes = new PropertyChangeSupport(this);
    
  public CalendarWidget() {
    format = DateFormat.getDateInstance(DateFormat.LONG);
    text = new TextField(format.format(date));
    text.setEditable(false);
    text.addActionListener(this);
    add(text);
    change = new Button("...");
    change.addActionListener(this);
    add(change);
  }

  public synchronized Date getDate() {
    return date;
  }
    
  public void setDate(Date newDate) {
    Date oldDate = date;
    date = newDate;
    text.setText(format.format(date));
    if(!change.isEnabled()) 
      change.setEnabled(true);    
    changes.firePropertyChange("date", oldDate, newDate);
  }
    
  public void actionPerformed(ActionEvent e) {
    if(e.getSource() == change) {
      change.setEnabled(false);
      calendar = new CalendarFrame(date, this);
      calendar.setLocation(new Point(getLocationOnScreen().x, 
        getLocationOnScreen().y + getSize().height));
      calendar.show();
    }
    if(e.getSource() == text) {
      try {
	date = format.parse(text.getText());
      } catch (Exception x) {
	System.out.println("Incorrect date format");
	text.setText(format.format(date));
      }
    }
  }

  public void addPropertyChangeListener(PropertyChangeListener l) {
      changes.addPropertyChangeListener(l);
  }    

  public void removePropertyChangeListener(PropertyChangeListener l) {
      changes.removePropertyChangeListener(l);
  }

  public static void main(String argv[]) {
    CalendarWidget widget = new CalendarWidget();
  }
};

class CalendarFrame extends Frame implements Serializable, ActionListener, 
  ItemListener {
    CalendarWidget parent;
    Choice month;
    TextField year;
    Panel days, monthNyear;
    int currentDay;
    String months[] = {"January", "February", "March", "April", "May", "June", 
		       "July", "August", "September", "October", "November",
		       "December"};
    int monthLength[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
    String weekdays[] = {"S", "M", "T", "W", "T", "F", "S"};
    DayCell[] cells;
    Dimension cellsize;
    int pad = 3;

  public CalendarFrame(Date date, CalendarWidget widget) {
    setVisible(false);
    parent = widget;
    setResizable(false);
    Calendar calendar = Calendar.getInstance();
    monthNyear = new Panel();
    month = new Choice();
    for (int i = 0; i < 12; i++)
      month.addItem(months[i]);
    month.addItemListener(this);
    year = new TextField(4);
    year.addActionListener(this);
    monthNyear.add(month);
    monthNyear.add(year);
    add(monthNyear, "North");
    days = new Panel(); 
    days.setLayout(new GridLayout(0, 7));
    
    setDate(date);
    arrangeDays();
    add(days, "South");
    pack();
  }
    
  public Date getDate() {
    Calendar calendar = Calendar.getInstance();
    calendar.set(getYear(), getMonth(), getDay());
    return calendar.getTime();
  }

  public void setDate(Date newDate) {
    Calendar calendar = Calendar.getInstance();
    calendar.setTime(newDate);
    setYear(calendar.get(Calendar.YEAR));
    setMonth(calendar.get(Calendar.MONTH));
    setDay(calendar.get(Calendar.DAY_OF_MONTH));
  }

  public int getYear() {
    return(Integer.parseInt(year.getText()));
  }

  public void setYear(int newYear) {
    year.setText(Integer.toString(newYear)); 
  }

  public int getMonth() {
    return(month.getSelectedIndex());
  }

  public void setMonth(int newMonth) {
    month.select(newMonth);
  }

  public int getDay() {
    return currentDay;
  }

  public void setDay(int newDay) {
     currentDay = newDay;
  }

  public void actionPerformed(ActionEvent e) {
    arrangeDays();
  }

  public void itemStateChanged(ItemEvent e) {
    arrangeDays();
  }

  public void setDateAndExit(int newDay) {
    cells[currentDay - 1].deactivate();
    setDay(newDay);
//    parent.setDate(getDate());
    Class args[] = {Date.class};
    try {
      Method m = CalendarWidget.class.getDeclaredMethod("setDate", args);
      Object objs[] = {getDate()};
      m.invoke(parent, objs);        
    } catch (Exception x) {
      System.out.println("Method invocation failed");
    }
    dispose();
  }

  private void arrangeDays() {
    days.removeAll();
    Font f = new Font("TimesRoman", Font.PLAIN, 10);
    setFont(f);
    FontMetrics fm = getFontMetrics(f);
    cellsize = new Dimension(fm.charWidth('0') * 2 + pad * 3, 
			     fm.getHeight() + pad * 2);
    int cMonth = getMonth();
    int cYear = getYear();
    Calendar calendar = Calendar.getInstance();
    calendar.set(cYear, cMonth, 1);
    int weekday = calendar.get(Calendar.DAY_OF_WEEK );

    int max;
    if((cMonth == 1) && (cYear % 4 == 0)) {
      max = 29;
    } else {
      max = monthLength[cMonth];
    }
//    int firstDayOfWeek = calendar.getFirstDayOfWeek();
    for (int i = 0; i < 7; i++) 
      days.add(new WeekdayCell(weekdays[i], cellsize,
			       this));
    for (int i = 1; i < weekday ; i++) 
      days.add(new WeekdayCell("", cellsize, this));
    cells = new DayCell[max];
    for (int i = 0; i < max; i++)
      days.add(cells[i] = new DayCell(i + 1, cellsize, this));
    cells[currentDay - 1].activate();
    pack();
  }
};

class DayCell extends Canvas implements Serializable {
  CalendarFrame parent;
  int day;
  int pad;
  Color active = Color.red;
  Color passive;

  public DayCell(int newDay, Dimension size, CalendarFrame frame) {
    parent = frame;
    day = newDay;
    addMouseListener( new Mouser());
    setSize(size);
    setFont(parent.getFont());
    pad = parent.pad;
  }

  public void paint(Graphics g) {
    if (day != 0) {
      FontMetrics fm = getFontMetrics(getFont());
      g.drawString(Integer.toString(day), pad, pad + fm.getHeight());
    }
  }

  public void activate() {
    passive = getBackground();
    setBackground(active);
    repaint();
  }

  public void deactivate() {
    setBackground(passive);
    repaint();
  }

  int getDay() {
    return (day);
  }

  class Mouser extends MouseAdapter {
    public void mouseClicked(MouseEvent e) {
      activate();
      parent.setDateAndExit(getDay());
    }
  }

  public Mouser mouser() {
    return new Mouser();
  }
};

class WeekdayCell extends Canvas implements Serializable {
  String day;
  int pad;

  public WeekdayCell(String newDay, Dimension size, CalendarFrame parent) {
    day = newDay;
    setSize(size);
    setFont(parent.getFont());
    pad = parent.pad;
  }

  public void paint(Graphics g) {
    FontMetrics fm = getFontMetrics(getFont());
    g.drawString(day, pad, pad + fm.getHeight());
  }
};

