import java.awt.*;
import java.io.*;
import java.util.*;

/*
 * MiniEdit - Copyright 1997 John Jensen.  All Rights Reserved.  See license.txt
 */

// ----------
// Notes:
//
// MiniEdit is an application.  You have to run it from a command line by typing
// "java MiniEdit" or "jview MiniEdit" or by assoicating the MiniEdit.class icon
// to the java interpreter by a method appropriate to your operating system.
// 
// This application uses some basic Java classes (TextArea, FileDialog,
// FileInputStream, FileOutputStream, etc.) to implement a simple NotePad
// or TeachText style text editor.
// 
// I think it worked out fairly well.  I had to do work-arounds for the modal
// dialog bug on Windows, and for the ".*.*" problem in FileDialog on that same
// platform.
// 
// I didn't put in a limit on file size, but things definitely start to lag when
// editing files greater than 100k or so.
// 
// The fun part is that I've run this same program on Solaris, AIX, and Windows.
// ----------

// To get a window MiniEdit extends Frame.  As part of a modal dialog fix, it
// implements NoteListener, a minimal interface for that purpose.

public class MiniEdit extends Frame implements NoteListener
{
	// The NoteListen member fuction of NoteListener is overridded to support
	// the following actions.

	final private int NOACTION	= 0;
	final private int NEWFILE	= 1;
	final private int OPENFILE	= 2;
	final private int QUITAPP	= 3;

	// size of internal input and output buffer

	final private int BUFSIZE	= 5000;

	// allocate standard components of MiniEdit as part of initialization:

	private TextArea textArea	= new TextArea();

	private MenuItem newItem	= new MenuItem("New");
	private MenuItem openItem	= new MenuItem("Open");
	private MenuItem saveItem	= new MenuItem("Save");
	private MenuItem saveAsItem	= new MenuItem("Save As");
	private MenuItem quitItem	= new MenuItem("Quit");

	private File file			= new File("Untitled");

	private String openPrompt	= new String("Open text file");
	private String closePrompt	= new String("Save file as");

	// initialize some state flags

	private boolean neverNamed	= true;
	private boolean dirty		= false;
	private boolean menuBusy	= false;

	// constructor puts together UI

	public MiniEdit()
	{
		this.setTitle();

		MenuBar mbar = new MenuBar();
		Menu m = new Menu("File");
		m.add(newItem);
		m.add(openItem);
		m.add(saveItem);
		m.add(saveAsItem);
		m.addSeparator();
		m.add(quitItem);            
		mbar.add(m);
		setMenuBar(mbar);

		add("Center", textArea);
	}

	// add current filename to window title

	private void setTitle()
	{
		super.setTitle("MiniEdit - " + file.getName());
	}

	// action handler, part of std java event handling

	public boolean action(Event evt, Object arg)
	{
		if (evt.target instanceof MenuItem)
		{
			if (evt.target == newItem)
			{
				if (isItSafe(NEWFILE))	// picture Olivier saying this
					newFile();
			}
			else
				if (evt.target == openItem)
			{
				if (isItSafe(OPENFILE))	// picture Olivier saying this
					openFile();
			}
			else
				if (evt.target == saveItem)
				saveFile(false);
			else
				if (evt.target == saveAsItem)
				saveFile(true);
			else
				if (evt.target == quitItem)
			{
				if (isItSafe(QUITAPP))	// picture Olivier saying this
					System.exit(0);
			}
		}
		else
			return super.action(evt, arg);

		return true;
	}

	// if the file has been modified (dirty), fire off a notify dialog
	// and pass it the "action" you want returned to noteListen.  This
	// dataflow beats the "modal dialog problem" on Windows

	// I believe isItSafe is what Laurence Olivier said repeatedly to
	// Dustan Hoffman in Marathon Man.

	private boolean isItSafe(int action)
	{
		if (dirty)
		{
			notify( "The current file has not been saved. ", action );
			return false;
		}
		else
			return true;
	}

	// implementation of the NoteListener interface - it does whatever
	// action the user agreed to by pressing "Continue" on a modal dialog

	public void noteListen(int action)
	{
		switch (action) 
		{
		case NOACTION:
			break;
		case NEWFILE:
			newFile();
			break;
		case OPENFILE:
			openFile();
			break;
		case QUITAPP:
			System.exit(0);
			break;
		}
	}

	// handleEvent is the other standard part of Java event handling

	public boolean handleEvent(Event evt)
	{
		if ((evt.id == Event.WINDOW_DESTROY) && (evt.target == this))
			if (isItSafe(QUITAPP))	// picture Olivier saying this
				System.exit(0);

		return super.handleEvent(evt);
	}

	// pick up keyDown wevents soley to mark the document as dirty.

	public boolean keyDown(Event  evt, int  key)
	{
		dirty = true;
		return false;
	}

	// clear out the document

	private void newFile()
	{
		neverNamed = true;
		dirty = false;
		file = new File("Untitled");
		this.setTitle();
		textArea.setText("");
		repaint();
	}

	// call getFile to get a file path, and if you get one open it

	private void openFile()
	{
		byte buf[];
		String data = new String("");

		int max,count,total;

		buf = new byte[BUFSIZE];

		if (getFile(FileDialog.LOAD))
		{
			try
			{
				FileInputStream input = new FileInputStream(file);
				max = input.available();
				total = 0;

				while (total < max)
				{
					count = BUFSIZE;
					if (max-total < count)
						count = max-total;
					count = input.read(buf,0,count);
					data += new String(buf,0,0,count);
					total += count;
				}

				input.close();
				neverNamed = false;
				dirty = false;
				this.setTitle();
				textArea.setText(data);
			}

			catch (IOException e)
			{
				notify("Error - could not read file", NOACTION);
			}
		}
		repaint();
	}

	// call getFile to get a file path, and if you get one save to it

	private void saveFile(boolean As)
	{
		byte buf[];
		String all;
		boolean ok;

		int max,count,total;

		buf = new byte[BUFSIZE];

		if (neverNamed || As)
		{
			ok = getFile(FileDialog.SAVE);
			neverNamed = false;
		}
		else
			ok = true;

		if (ok)
		{
			String data = textArea.getText();
			try
			{
				FileOutputStream output = new FileOutputStream(file);
				max = data.length();
				total = 0;

				while (total < max)
				{
					count = BUFSIZE;
					if (max-total < count)
						count = max-total;
					data.getBytes(total, total+count, buf, 0);
					output.write(buf,0,count);
					total += count;
				}

				output.close();
				neverNamed = false;
				dirty = false;
				this.setTitle();
			}

			catch (IOException e)
			{
				notify("Error - could not write file", NOACTION);
			}
		}
		repaint();
	}

	// use FileDialog to get a filepath

	private boolean getFile(int mode)
	{
		String prompt;
		String filename;
		String pathname;

		if (mode == FileDialog.LOAD)
			prompt = openPrompt;
		else
			prompt = closePrompt;

		FileDialog d = new FileDialog(this, prompt, mode);

		if (mode == FileDialog.LOAD)
			d.setFile("*");
		else
			d.setFile(file.getName());

		d.setDirectory(".");
		d.show();

		filename = d.getFile(); 

		if (filename.endsWith(".*.*")) // must be a java-on-windows bug
			filename = filename.substring(0,filename.length() - 4);

		pathname = d.getDirectory() + filename; 

		if (filename != null)
		{
			file = new File(pathname);
			return(true);
		}
		else
			return(false);
	}

	// launch a note

	private void notify(String s, int action)
	{
		Note note = new Note(this,s,action);
		note.show();
	}

	// main declares the MiniEdit frame which does everything

	public static void main(String[] args)
	{
		Frame f = new MiniEdit();
		f.resize(500, 400);
		f.show();  
	}
}
