/*
 * Copyright (c) 1997 John Jensen. All rights reserved.
 *
 * This software is FREE FOR COMMERCIAL AND NON-COMMERCIAL USE,
 * provided the following condition is met.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that any copy or derivative of this software or documentation
 * retaining the name "John Jensen" also retains this condition and the
 * following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * CopyrightVersion 1.0
 */

import java.awt.*;
import java.io.*;
import java.util.*;

class DocMan
{
	private mpEDIT		mpEdit;			// the boss
	private Properties	strings;
	private PropMan		propMan;		// the user preferences

	private Vector			frames;		// the document frames
	private Vector			textMenus;	// attached edit menu handlers
	private Vector			lines;		// the document data
	private File			file;		// the document file
	private JournalItem		undo_list;	// where we are in undo
	private JournalItem		redo_list;	// where we are in redo

	private	boolean dirty;
	private	boolean neverNamed;

	private String	lineSeparator;

	private final int REPLACE_LINE	= 1;
	private final int SPLIT_LINE	= 2;
	private final int JOIN_LINE		= 3;
	private final int INSERT		= 4;
	private final int DELETE		= 5;

	public DocMan(mpEDIT mpe, Properties str, PropMan pm)
	{
		lines = new Vector(100,100);
		frames = new Vector(4,4);
		textMenus = new Vector(4,4);

		lines.addElement(new String(""));

		mpEdit = mpe;
		strings = str;
		propMan = pm;

		lineSeparator = propMan.getLineSeparator();

		undo_list = redo_list = null;
	}

	public void setUntitled(int count)
	{
		String filename = new String(strings.getProperty("UntitledDoc")+count);
		file = new File(filename);
		setTitles();
		neverNamed = true;
	}

	public void setTitles()
	{
		int i,max;
		String name = file.toString();

		max = frames.size();

		for (i = 0; i < max; i++ )
		{
			if (max > 1)
				name = new String(file.toString() + ":" + (i+1));
				
			((TextFrame)frames.elementAt(i)).setTitle(name);
		}
	}

	public TextFrame newFrame(Point point, Dimension size)
	{
		TextFrame textFrame = new TextFrame(mpEdit, strings, propMan, this);
		textFrame.setLocation(point);
		textFrame.setSize(size);
		frames.addElement(textFrame);
		updateMenus();
		return textFrame;
	}

	public void addTextMenu(TextMenu e)
	{
		textMenus.addElement(e);
	}

	public void closeFrame(TextFrame textFrame)
	{
		int max;
		
		max = frames.size();

		if ((max > 1) || !dirty)
		{
			closeOneFrame(textFrame);
		}
		else
		{
			DirtyDialog ab = new DirtyDialog(this,
											 textFrame,
											 strings,
											 strings.getProperty("DialogDirty"),
											 file.getName());
			ab.show();
		}
	}

	public void closeOneFrame(TextFrame textFrame)
	{
		int i;
		int max;
		
		max = frames.size();

		for (i = 0; i < max; i++ )
		{
			if (textFrame == (TextFrame)frames.elementAt(i))
			{
				frames.removeElementAt(i);
				textFrame.dispose();
				max--;
				break;
			}
		}

		if (max == 0)
			mpEdit.closeDoc(this);
		else
			setTitles();
	}

	public void updateUndoItems(boolean foo,boolean bar)
	{
		int i,max;

		max = textMenus.size();
		for (i = 0; i < max; i++ )
		{
			TextMenu e = (TextMenu)textMenus.elementAt(i);
			e.updateUndoItems(foo, bar);		
		}
	}

	public void newDoc()
	{
		int i,max;
		lines.removeAllElements();
		lines.addElement(new String(""));
		max = frames.size();
		for (i = 0; i < max; i++ )
		{
			TextFrame f = (TextFrame)frames.elementAt(i);
			f.clearCanvas();		
			f.redoCanvas();
		}
		setUntitled(mpEdit.getUntitled());
		clear_undo();
		updateMenus();
	}

	public boolean isBusy()
	{
		return dirty || !neverNamed;
	}

	public void openDoc(TextFrame textFrame, String filename)
	{
		int i,max;

		file = new File(filename);
		if (file.isFile())
		{
			neverNamed = false;
			max = frames.size();
			for (i = 0; i < max; i++ )
				((TextFrame)frames.elementAt(i)).clearCanvas();
			setTitles();
			read(textFrame,filename);
			for (i = 0; i < max; i++ )
				((TextFrame)frames.elementAt(i)).redoCanvas();
		}

		textFrame.getCanvas().requestFocus();
	}

	public boolean fileSave(TextFrame textFrame)
	{
		return doSaveFile(textFrame, false);
	}

	public void fileSaveAs(TextFrame textFrame)
	{
		doSaveFile(textFrame, true);
	}

	private boolean doSaveFile(TextFrame textFrame, boolean As)
	{
		boolean ok = true;
		String filename = null;

		if (neverNamed || As)
		{
			filename = getFileForSave(textFrame);
			if (filename == null)
			{
				ok = false;
			}
			else
			{
				file = new File(filename);
				setTitles();
				neverNamed = false;
			}
		}

		if (ok)
			write(textFrame,file);

		return ok;
	}

	private String getFileForSave(TextFrame textFrame)
	{
		String prompt;
		String filename;
		String pathname;

		prompt = strings.getProperty("DialogSaveAs");

		FileDialog d = new FileDialog(textFrame, prompt, FileDialog.SAVE);

		d.setFile(file.getName());
		d.setDirectory(file.getPath());

		d.setVisible(true);

		filename = d.getFile(); 

		pathname = d.getDirectory() + filename; 

		d.dispose();

		if (filename != null)
			return pathname;
		else
			return null;
	}

	private final int SLOW_READ  = 2000;
	private final int SLOW_WRITE = 10000;

	public void read(TextFrame textFrame, String filename)
	{
		Progress readProgress = null;
		boolean show_progress;
		long max,part,all,tens;
		String line;

		max = file.length();
		tens = max / 10;
		part = all = 0;
		show_progress = max > SLOW_READ;

		if (show_progress)
		{
			if (readProgress == null)
				readProgress = new Progress(textFrame, "Reading file ... ");

			readProgress.show();
		}

		lines.removeAllElements();

		try
		{
			FileReader fr = new FileReader(filename);
			BufferedReader br = new BufferedReader(fr);

			while ( (line=br.readLine()) != null)
			{
				if (show_progress)
				{
					part += line.length();
					if (part > tens)
					{
						all += part;
						readProgress.update( (int)(100 * all / max) );
						part = 0;
					}
				}
				lines.addElement(line);
			}

			br.close();
			fr.close();
		}

		catch (IOException e)
		{
			System.out.println("Error - could not read file");
		}
		if (readProgress != null)
			readProgress.dispose();
		if (lines.size() < 1)
			lines.addElement(new String(""));
	}

	public void write(TextFrame textFrame, File file)
	{
		Progress writeProgress = null;
		boolean show_progress;
		long max,part,all,tens;
		String line;

		max = lines.size();
		tens = max / 10;
		part = all = 0;
		show_progress = max > SLOW_WRITE;

		if (show_progress)
		{
			if (writeProgress == null)
				writeProgress = new Progress(textFrame, "Writing file ... ");

			writeProgress.show();
		}

		try
		{
			FileWriter fw = new FileWriter(file);
			BufferedWriter bw=new BufferedWriter(fw);

			for (int i=0;i<max;i++)
			{
				line = (String)lines.elementAt(i);
				bw.write(line,0,line.length());
				bw.newLine();
				if (show_progress)
				{
					part++;
					if (part > tens)
					{
						all += part;
						writeProgress.update( (int)(100 * all / max) );
						part = 0;
					}
				}
			}

			bw.close();
			fw.close();

			dirty = false;
		}

		catch (IOException e)
		{
			System.out.println("Error - could not write file");
		}
		if (writeProgress != null)
			writeProgress.dispose();
	}


	public void clear_undo()
	{
		undo_list = redo_list = null;			
	}

	public void updateFrames(int first, int last)
	{
		int i;
		TextFrame textFrame;
		TextCanvas textCanvas;
		
		i = frames.size();

		while (i>0)
		{
			i--;
			textFrame = (TextFrame)frames.elementAt(i);
			textCanvas = textFrame.getCanvas();
			textCanvas.legalizeCursor();
			textCanvas.linesChanged(first,last);
		}
	}

	private void legalizeCursors()
	{
		int i,max;

		max = frames.size();
		for (i = 0; i < max; i++ )
		{
			TextFrame f = (TextFrame)frames.elementAt(i);
			f.legalizeCursor();
		}
	}

	public int getMax()
	{
		return lines.size();
	}

	public String getLine(int i)
	{
		return (String)lines.elementAt(i);
	}

	private void updateMenus()
	{
		boolean got_undo;
		boolean got_redo;

		got_undo = undo_list != null;
		got_redo = redo_list != null;

		updateUndoItems(got_undo, got_redo);
	}

	public void undo(TextFrame textFrame)
	{
		JournalItem temp;
		TextPosition tp;

		if (undo_list != null)
		{
			temp = undo_list;
			undo_list = temp.next;

			switch (temp.action)
			{
			case REPLACE_LINE:
				redo_line(temp);
				break;
			case SPLIT_LINE:
				split_or_join(temp,true);
				break;
			case JOIN_LINE:
				split_or_join(temp,false);
				break;
			case INSERT:
				copy_or_cut(temp,true);
				break;
			case DELETE:
				insert(temp);
				break;
			default:;
			}

			tp = new TextPosition(temp.line,temp.column);
			textFrame.setPos(tp);

			legalizeCursors();

			temp.next = redo_list;
			redo_list = temp;
		}

		updateMenus();
	}

	public void redo(TextFrame textFrame)
	{
		JournalItem temp;
		TextPosition tp;
		int line, column;

		if (redo_list != null)
		{
			temp = redo_list;
			redo_list = temp.next;

			line = temp.line;
			column = temp.column;

			switch (temp.action)
			{
			case REPLACE_LINE:
				column = temp.text.length();
				redo_line(temp);
				break;
			case SPLIT_LINE:
				split_or_join(temp,false);
				column = 0;
				break;
			case JOIN_LINE:
				split_or_join(temp,true);
				break;
			case INSERT:
				insert(temp);
				line = temp.eline;
				column = temp.ecolumn;
				break;
			case DELETE:
				copy_or_cut(temp,true);
				break;
			default:;
			}

			tp = new TextPosition(line,column);
			textFrame.setPos(tp);

			legalizeCursors();

			temp.next = undo_list;
			undo_list = temp;
		}

		updateMenus();
	}

	public void insert_char(int line, int column, char c)
	{
		String  s;
		s = (String)lines.elementAt(line);
		remember_line(line,column,s);
		s = s.substring(0, column) + c + s.substring(column, s.length());
		lines.setElementAt(s,line);
		updateFrames(line,line);
		dirty = true;
	}

	public void delete_char(int line, int column)
	{
		String s;
		s = (String)lines.elementAt(line);
		remember_line(line,column,s);
		s = s.substring(0, column) + s.substring(column+1, s.length());
		lines.setElementAt(s,line);
		updateFrames(line,line);
		dirty = true;
	}

	private void remember_line(int line, int column, String s)
	{
		boolean new_line = true;

		if (undo_list != null)
			new_line = (undo_list.action != REPLACE_LINE) || (undo_list.line != line);

		if (new_line)
		{
			JournalItem new_journal = new JournalItem();
			new_journal.action = REPLACE_LINE;
			new_journal.line = line;
			new_journal.column = column;
			new_journal.text = new String(s);
			new_journal.next = undo_list;
			undo_list = new_journal;
		}

		redo_list = null;
		updateMenus();
	}

	private void redo_line(JournalItem i)
	{
		String s;
		int line = i.line;
		s = (String)lines.elementAt(line);
		lines.setElementAt(i.text,line);
		i.text = s;

		updateFrames(line,line);
	}

	public void split_line(int line, int column)
	{
		JournalItem new_journal = new JournalItem();
		new_journal.action = SPLIT_LINE;
		new_journal.line = line;
		new_journal.column = column;
		split_or_join(new_journal,false);
		new_journal.next = undo_list;
		undo_list = new_journal;
		redo_list = null;
		updateMenus();
		updateFrames(line,line+1);
		dirty = true;
	}

	public void join_line(int line, int column)
	{
		JournalItem new_journal = new JournalItem();
		new_journal.action = JOIN_LINE;
		new_journal.line = line;
		new_journal.column = column;
		split_or_join(new_journal,true);
		new_journal.next = undo_list;
		undo_list = new_journal;
		redo_list = null;
		updateMenus();
		updateFrames(line,line+1);
		dirty = true;
	}

	public void split_or_join(JournalItem i, boolean join)
	{
		String s;
		int line = i.line;
		int column = i.column;

		if (join)
		{
			s = (String)lines.elementAt(line);
			s = s.concat((String)lines.elementAt(line+1));
			lines.setElementAt(s,line);
			lines.removeElementAt(line+1);
		}
		else
		{
			s = (String)lines.elementAt(line);
			lines.setElementAt(s.substring(0,column),line);
			lines.insertElementAt(s.substring(column,s.length()),++line);
		}
	}

	public TextPosition insert_section(int line, int column, String s)
	{
		int charCt;
		int saveCt;
		int charMax;
		char c,c2;
		String s2 = null;
		TextPosition tp;

		// start creating journal item

		JournalItem new_journal = new JournalItem();
		new_journal.action = INSERT;
		new_journal.text = new String(s);
		new_journal.line = line;		// starting line and column
		new_journal.column = column;

		// insert the text

		tp = insert(new_journal);
	
		// finish creating journal entry

		new_journal.eline = tp.line;		// updated line and column
		new_journal.ecolumn = tp.column;
		new_journal.next = undo_list;
		undo_list = new_journal;
		redo_list = null;
		updateMenus();
		updateFrames(line,tp.line);
		dirty = true;

		return tp;
	}

	public TextPosition insert(JournalItem i)
	{
		int line = i.line;
		int column = i.column;
		String s = i.text;

		int charCt;
		int saveCt;
		int charMax;
		char c,c2;
		String s2 = null;

		// insert the text

		charMax = s.length();
		charCt = saveCt = 0;
		
		while (charCt < charMax)
		{
			c = s.charAt(charCt);
			charCt++;
			if ((c == '\r') || (c == '\n'))
			{
				s2 = (String)lines.elementAt(line);
				lines.setElementAt(s2.substring(0,column) + s.substring(saveCt,charCt-1),line);
				lines.insertElementAt(s2.substring(column,s2.length()),++line);
				column = 0;
				if (charCt < charMax)
				{
					c2 = s.charAt(charCt);
					if (((c == '\r') && (c2 == '\n')) || ((c2 == '\r') && (c == '\n')))
						charCt++;
				}
				saveCt = charCt;
			}
		}
		
		if (saveCt < charCt)
		{
			s2 = (String)lines.elementAt(line);
			s = s.substring(saveCt,charCt);
			if (column == 0)
				s2 = s + s2;
			else
				s2 = s2.substring(0, column) + s + s2.substring(column, s2.length());
			column += s.length();
			lines.setElementAt(s2,line);
		}
		return new TextPosition(line,column);
	}

	public String delete_section(int line, int column, int eline, int ecolumn, boolean cut)
	{
		String text;

		JournalItem new_journal = new JournalItem();
		new_journal.action = DELETE;
		new_journal.line = line;
		new_journal.column = column;
		new_journal.eline = eline;
		new_journal.ecolumn = ecolumn;

		text = copy_or_cut(new_journal,cut);

		if (cut)
		{
			new_journal.text = new String(text);
			new_journal.next = undo_list;
			undo_list = new_journal;
			redo_list = null;
			updateMenus();
			dirty = true;
		}

		return text;
	}

	private String copy_or_cut(JournalItem i, boolean cut)
	{
		String s,s2;
		String text = null;

		int line = i.line;
		int column = i.column;
		int eline = i.eline;
		int ecolumn = i.ecolumn;
	
		if (line == eline)
		{
			s = (String)lines.elementAt(line);
			text = s.substring(column,ecolumn);
			if (cut)
			{
				s = s.substring(0, column) + s.substring(ecolumn, s.length());
				lines.setElementAt(s,line);
			}
		}
		else
		{
			s = (String)lines.elementAt(line);
			text = s.substring(column,s.length());
			s = s.substring(0, column);

			int diff = eline - line;
			int k = line + 1;

			for (int j = 1; j < diff; j++)
			{
				s2 = (String)lines.elementAt(k);
				text = text + lineSeparator + s2;
				if (cut)
					lines.removeElementAt(k);
				else
					k++;
			}

			if (k != lines.size())
    		{
				s2 = (String)lines.elementAt(k);
				text = text + lineSeparator + s2.substring(0,ecolumn);
				s = s + s2.substring(ecolumn,s2.length());
				if (cut)
					lines.removeElementAt(k);
			}
			if (cut)
				lines.setElementAt(s,line);
		}
		return text;
	}
}	

class JournalItem
{
	public JournalItem		next;
	public String			text;
	public int				action;
	public int				line;
	public int				column;
	public int				eline;
	public int				ecolumn;
}