/*
 * Copyright (c) 1997 John Jensen. All rights reserved.
 *
 * This software is FREE FOR COMMERCIAL AND NON-COMMERCIAL USE,
 * provided the following condition is met.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that any copy or derivative of this software or documentation
 * retaining the name "John Jensen" also retains this condition and the
 * following disclaimer.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * CopyrightVersion 1.0
 */

import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;

class	TextCanvas
extends		Canvas
implements	MouseMotionListener,
			MouseListener,
			ComponentListener,
			AdjustmentListener,
			KeyListener,
			FocusListener
{
	final private int EDGE = 10;

	private Scrollbar	horiz;				// control
	private Scrollbar	vert;				// control

	private boolean		eactive,hactive;	// is there currently a range?
	private boolean		mouseDown;
	
	private int			sx,sy,ny;			// current scroll positions
	private int			line,column,pix;	// the current column
	private int			eline,ecolumn,epix;	// the current end column
	private int			oldlines;			// repaint to old line count (if longer)
	private int			fontHeight;			// save time
	private int			fontDescent;		// save time
	private int			tabSize = 4;		// preference

	private Font		font;				// preference
	private FontMetrics fontMetrics;		// save time
	private TextFrame	textFrame;			// friendly object
	private TextCursor	textCursor;			// friendly object
	private TextMenu	textMenu;			// friendly object
	private Dimension	dimension;			// save time
	private DocMan		docMan;				// the document
	private PropMan		propMan;			// the preferences

	private TextScroller textScroller = null;
	
	public TextCanvas(PropMan pm, DocMan doc, TextFrame tf, Scrollbar h, Scrollbar v)
	{
		super();

		propMan = pm;
		docMan = doc;

		setBackground(Color.white);
		setForeground(Color.black);

		textFrame = tf;

		horiz = h;
		horiz.addAdjustmentListener(this);
		horiz.addKeyListener(this);
		
		vert = v;
		vert.addAdjustmentListener(this);
		vert.addKeyListener(this);

		addComponentListener(this);
		addMouseListener(this);
		addKeyListener(this);
		addFocusListener(this);

		setFont();

		clear();
		validate();
		requestFocus();

		textCursor = new TextCursor(this);
		textCursor.start();
	}

	private boolean gotFocus = false;

	public void focusGained(FocusEvent e)
	{
		gotFocus = true;
		release_cursor();
	}

	public void focusLost(FocusEvent e)
	{
		pause_cursor();
		gotFocus = false;
	}

	public void release_cursor()
	{
		if ((textCursor != null) && !eactive && gotFocus)
			textCursor.release_cursor(true);
	}

	public void pause_cursor()
	{
		if ((textCursor != null) && !eactive && gotFocus)
			textCursor.pause_cursor(true);
	}

	public void setTextMenu(TextMenu tm)
	{
		textMenu = tm;
	}

	public void clear()
	{
		clear_area(null);
		hactive = eactive = false;
		if (textMenu != null)
		{
			textMenu.updateCopyItems(false);
		}
		oldlines = line = column = sx = sy = 0;
		pix = EDGE;
		horiz.setValue(0);
		vert.setValue(0);
	}

	public void setFont()
	{
		Properties properties = propMan.getProperties();

		String name,style,size;
		
		name = properties.getProperty("mpEDIT.font.name");     // add defaults
		style = properties.getProperty("mpEDIT.font.style");
		size = properties.getProperty("mpEDIT.font.size");

		font = new Font(name,
						Integer.valueOf(style).intValue(),
						Integer.valueOf(size).intValue());

		Graphics g = getGraphics();
		
		if (g != null)
		{
			clear_area(g);
			updateFonts(g);
			resizeLines();
			redoCanvas();
			repaint();
			g.dispose();
		}
		else
			fontMetrics = null;
	}

	private void clear_area(Graphics g)
	{
		boolean foobar = false;
		
		pause_cursor();
			
		if (g == null)
		{
			g = getGraphics();
			foobar = true;
		}
		
		if (g != null)
			g.clearRect(0,0,dimension.width,dimension.height);

		if (foobar && (g != null))
			g.dispose();
	}
			
	public void setTab(int tab)
	{
		clear_area(null);
		tabSize = tab;
		resizeLines();
		repaint();
	}

	public void setPos(TextPosition tp)
	{
		hactive = eactive = false;
		line = eline = tp.line;
		column = ecolumn = tp.column;
		pix = epix = pix_at(line,column);
		shiftVert(line);
		shiftHoriz(pix);
		repaint();
	}

	public void legalizeCursor()
	{
		String whole;
		boolean illegal = false;
		int max_line;
		int max_column;

		max_line = docMan.getMax();

		if (line >= max_line)
		{
			line = max_line - 1;
			illegal = true;
		}
		
		whole = docMan.getLine(line);
		max_column = whole.length();

		if (column > max_column)
		{
			column = max_column;
			illegal = true;
		}

		if (eactive && (eline >= max_line))
		{
			illegal = true;
		}

		if (eactive && !illegal)
		{
			whole = detabbed(docMan.getLine(eline));
			max_column = whole.length();

			if (ecolumn > max_column)
			{
				illegal = true;
			}
		}

		if (line < 0)
			line = 0;

		if (column < 0)
			column = 0;

		if (illegal)
		{
			hactive = eactive = false;
			pix = epix = pix_at(line,column);
			shiftVert(line);
			shiftHoriz(pix);
			repaint();
		}
	}

	private void updateFonts(Graphics g)
	{
		boolean foobar = false;
		
		if (g == null)
		{
			g = getGraphics();
			foobar = true;
		}
		
		if (g != null)
		{
			g.setFont(font);
			fontMetrics = g.getFontMetrics(font);
			fontHeight = fontMetrics.getHeight();
			fontDescent = fontMetrics.getDescent();
			
			if (foobar)
				g.dispose();
		}
	}

	public boolean find( String pattern )
	{
		int i,max,ct,start,from;
		String whole;

		pause_cursor();

		max = docMan.getMax();
		i = line;
		ct = 0;

		while (ct++ <= max)
		{
			whole = detabbed(docMan.getLine(i));

			if (( i == line ) && ( ct == 1 ))
			{
				if (eactive)
					from = column + 1;
				else
					from = column;
			}
			else
				from = 0;

			start = whole.indexOf(pattern,from);

			if (start >= 0)
			{
				line = eline = i;
				column = start;
				pix = stringLength(whole.substring(0,column)) + EDGE;
				ecolumn = start + pattern.length();
				epix = stringLength(whole.substring(0,ecolumn)) + EDGE;
				eactive = true;
				setup_h();
				save_h();
				if (textMenu != null)
					textMenu.updateCopyItems(eactive);
				shiftVert(line);
				shiftHoriz(pix);
				shiftHoriz(epix);
				repaint();
				return true;
			}

			i++;
			if (i >= max)
				i = 0;
		}

		release_cursor();
		return false;
	}

	public synchronized void mousePressed(MouseEvent e)
	{
		Graphics g;

		requestFocus();

		if (mouseDown)
			return;

		mouseDown = true;
		
		pause_cursor();
		g = getGraphics();
		updateFonts(g);

		clickPosition( e.getX(), e.getY() );

		if ((cline < 0) || (cline >= docMan.getMax()))
			return;

		eline = cline;
		ecolumn = ccolumn;
		epix = cpix;

		int clickCount = e.getClickCount();

		if (clickCount == 3)
		{
			eline = line = cline;
			column = 0;
			pix = EDGE;
			String whole = docMan.getLine(cline);
			ecolumn = whole.length();
			epix = stringLength(whole) + EDGE;
			eactive = true;
			setup_h();
			save_h();
			if (textMenu != null)
				textMenu.updateCopyItems(eactive);
			repaint();
		}
		else
		if (clickCount == 2)
		{
			char c;
			eline = line = cline;
			String whole = docMan.getLine(cline);
			column = ecolumn = ccolumn;
			while (true)
			{
				if (column == 0)
					break;

				if (!Character.isLetterOrDigit(whole.charAt(column-1)))
					break;

				column--;
			}
			pix = stringLength(whole.substring(0,column)) + EDGE;
			int max = whole.length();
			while (true)
			{
				if (ecolumn >= max)
					break;

				if (!Character.isLetterOrDigit(whole.charAt(ecolumn)))
					break;

				ecolumn++;
			}
			epix = stringLength(whole.substring(0,ecolumn)) + EDGE;
			eactive = true;
			setup_h();
			save_h();
			if (textMenu != null)
				textMenu.updateCopyItems(eactive);
			repaint();
		}
		else
		if (e.isShiftDown())
		{
			eline = cline;
			ecolumn = ccolumn;
			epix = cpix;
			eactive = true;
			setup_h();
			save_h();
			if (textMenu != null)
				textMenu.updateCopyItems(eactive);
			repaint();
		}
		else
		{
			if (hactive)
				flip_h( g, oline, opix, oeline, oepix );
			eline = line = cline;
			ecolumn = column = ccolumn;
			epix = pix = cpix;
			hactive = false;
			eactive = true;
			if (textMenu != null)
				textMenu.updateCopyItems(eactive);
			addMouseMotionListener(this);
		}
		
		g.dispose();
	}
	
	private int opix, oepix, oline, oeline;
	private int hpix, hepix, hline, heline, hcolumn, hecolumn;
	private int lastx,lasty;

	public synchronized void mouseDragged(MouseEvent e)
	{
		Graphics g;
		
		g = getGraphics();
		updateFonts(g);
		
		if (e != null)
		{
			lastx = e.getX();
			lasty = e.getY();
		}

		clickPosition( lastx, lasty );

		eline = cline;
		ecolumn = ccolumn;
		epix = cpix;

		eactive = ((eline != line) || (ecolumn != column));
		if (eactive || hactive)
		{
			setup_h();
			
			if ((eline < sy) || 
				(eline >= sy+ny) ||
				(epix < sx) ||
				(epix >= dimension.width))
			{
				if (textScroller == null)
				{
					textScroller = new TextScroller(this);
					textScroller.start();
				}
				shiftVert(eline);
				shiftHoriz(epix);
				setup_h();
				save_h();
				for (int i = 0; i < ny; i++)
					drawLine(g,sy+i);
			}
			else
			{
				if (textScroller != null)
				{
					textScroller.stop();
					textScroller = null;
				}
				if (hactive)
				{
					if ((hline < oline) || ((hline == oline) && (hpix < opix)))
						flip_h( g, hline, hpix, oline, opix );
					if ((hline > oline) || ((hline == oline) && (hpix > opix)))
						flip_h( g, oline, opix, hline, hpix );
					if ((heline < oeline) || ((heline == oeline) && (hepix < oepix)))
						flip_h( g, heline, hepix, oeline, oepix );
					if ((heline > oeline) || ((heline == oeline) && (hepix > oepix)))
						flip_h( g, oeline, oepix, heline, hepix );
				}
				else
					flip_h( g, hline, hpix, heline, hepix );
				
				hactive = eactive;
				save_h();
			}
		}	
		g.dispose();
	}

	private int cline, ccolumn, cpix;

	void clickPosition( int x, int y )
	{
		cline = (y / fontHeight) + sy;

		if (cline < 0)
		{
			cline = ccolumn = cpix = 0;
			return;
		}
		
		if (cline >= docMan.getMax())
		{
			cline = docMan.getMax() - 1;
			String whole = docMan.getLine(cline);
			ccolumn = whole.length();
			cpix = stringLength(whole) + EDGE;
			return;
		}

		x += sx;

		ccolumn = 0;
		cpix = 0;

		if (x > EDGE)
		{
			String whole = docMan.getLine(cline);
			String part;

			ccolumn = whole.length();
			
			if (( ccolumn > 0 ) && ( x < (cpix = stringLength(whole) + EDGE) ))
			{
				if (ccolumn > 10)
				{
					int delta = ccolumn / 5;

					do {
						ccolumn -= delta;
						if (ccolumn < 0)
						{
							ccolumn = 0;
							part = null;
						}
						else
							part = whole.substring(0,ccolumn);
					} while (( ccolumn > 0 ) && ( x < (cpix = stringLength(part) + EDGE) ));

					ccolumn += delta;
				}


				while (( ccolumn > 0 ) && ( x < (cpix = stringLength(whole) + EDGE) ))
					whole = whole.substring(0,--ccolumn);

				if (ccolumn == 0)
					cpix = EDGE;

			}
		}
	}

	private void flip_h(Graphics g, int sline, int spix, int eline, int epix)
	{
		int i;
		int bx,ex,by;
		
		g.setXORMode(Color.white);
		
		if (eline >= sy + ny)
		{
			eline = sy + ny - 1;
			epix = 5000;
		}
	
		for (i = sline; i <= eline; i++)
		{
			by = (i - sy) * fontHeight;
			bx = 0;
			ex = 5000;
			if (i == sline)
			{
				bx = spix - sx;
			}
			if (i == eline)
			{
				ex = epix - sx;
				if (sline == eline)
					ex -= bx;
			}
			g.fillRect(bx,by,ex,fontHeight);
		}
	}
		
	private void setup_h()
	{
		if ((line < eline) || ((line == eline) && (column <= ecolumn)))
		{
			hline = line;
			hcolumn = column;
			hpix = pix;
			heline = eline;
			hecolumn = ecolumn;
			hepix = epix;
		}
		else
		{
			hline = eline;
			hcolumn = ecolumn;
			hpix = epix;
			heline = line;
			hecolumn = column;
			hepix = pix;
		}
	}

	private void save_h()
	{
		opix = hpix;
		oepix = hepix;
		oline = hline;
		oeline = heline;
		hactive = eactive;
	}
		
	public synchronized void mouseReleased(MouseEvent e)
	{
		if (!mouseDown)
			return;

		mouseDown = false;
		
		if (textScroller != null)
		{
			textScroller.stop();
			textScroller = null;
		}
		removeMouseMotionListener(this);
		eactive = ((eline != line) || (ecolumn != column));
		release_cursor();
		if (textMenu != null)
			textMenu.updateCopyItems(eactive);
	}
	
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {}
	public void keyTyped(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}
	
	private final int myMask = KeyEvent.CTRL_MASK | KeyEvent.META_MASK | KeyEvent.ALT_MASK;

    public void keyPressed(KeyEvent e)
	{
		int		keyCode = e.getKeyCode();
		char	keyChar;
		int		max;
		String	s;

		if (!gotFocus)
			return;
		
		if ((e.getModifiers() & myMask) != 0)
			return;

		if ((keyCode == e.VK_UP) && (line == 0))
			return;

		pause_cursor();

		switch (keyCode) {
		case e.VK_UP:
			line--;
			cursorAdjust();
			return;
		case e.VK_DOWN:
			line++;
			cursorAdjust();
			return;
		case e.VK_RIGHT:
			column++;
			cursorAdjust();
			return;
		case e.VK_LEFT:
			column--;
			cursorAdjust();
			return;
		default:
			keyChar = e.getKeyChar();
			if (keyChar != KeyEvent.CHAR_UNDEFINED)
			{
				if (keyCode == KeyEvent.VK_ENTER)
				{
					if (hactive)
						copy(true,false);
					docMan.split_line(line,column);
					shiftVert(++line);
					column = 0;
					pix = EDGE;
					shiftHoriz(0);
					repaint();
					return;
				}
				else
				if (keyCode == KeyEvent.VK_BACK_SPACE)
				{
					if (hactive)
					{
						copy(true,true);
						return;
					}
					if (column > 0)
					{
						column--;
						docMan.delete_char(line,column);
						return;
					}
					else
					if (line > 0)
					{
						line--;
						s = docMan.getLine(line);
						column = s.length();
						shiftHoriz(stringLength(s) + EDGE);
						docMan.join_line(line,column);
						shiftVert(line);
						repaint();
						return;
					}
				}
				else
				if (keyCode == KeyEvent.VK_DELETE)
				{
					if (hactive)
					{
						copy(true,true);
						return;
					}
					s = docMan.getLine(line);
					max = s.length();
					if (column < max)
					{
						docMan.delete_char(line,column);
						return;
					}
					else
					if (line+1 < docMan.getMax())
					{
						docMan.join_line(line,column);
						repaint();
						return;
					}
				}
				else
				if (keyCode != KeyEvent.VK_ESCAPE &&
					keyChar != '\n' &&
					keyChar != '\b')
				{
					if (hactive)
						copy(true,false);
					column++;
					docMan.insert_char(line,column-1,keyChar);
					return;
				}
			}
		}
		release_cursor();
	}
	
	public void paste(String s)
	{
		int oldline;
		int oldcolumn;
		TextPosition tp;
		
		pause_cursor();

		if (hactive)
			copy(true,false);

		oldline = line;
		oldcolumn = column;

		tp = docMan.insert_section(line,column,s);
		line = tp.line;
		column = tp.column;

		pix = pix_at( line, column );
		shiftVert(line);
		shiftHoriz(pix);
		
		if (oldline != line)
		{
			repaint();
		}
	}

	int pix_at(int line, int column)
	{
		if (column == 0)
			return EDGE;
		else
			return stringLength((docMan.getLine(line)).substring(0, column)) + EDGE;
	}

	public String copy(boolean cut, boolean visible)
	{
		String s;
		int oldline;
		int oldcolumn;
		boolean paint = false;

		if (!hactive)
			return null;

		if (hline == heline)
		{
			oldline = hline;
			oldcolumn = hcolumn;
		}
		else
			paint = true;
		
		s = docMan.delete_section(hline, hcolumn, heline, hecolumn, cut);

		if (visible)
			docMan.updateFrames(hline,heline);

		if (cut)
		{
			line = hline;
			column = hcolumn;
			hactive = eactive = false;

			if (shiftVert(line))
				paint = true;

			pix = pix_at(line,column);
			if (shiftHoriz(pix))
				paint = true;

			if (paint)
			{
				repaint();
			}
			else
			{
				if (visible)
					linesChanged(line,line);
			}
		}

		return s;
	}
	
	private void cursorAdjust()
	{
		int min, max;
		boolean paint = false;
		
		if (eactive)
		{
			paint = true;
			hactive = eactive = false;
			if (textMenu != null)
				textMenu.updateCopyItems(eactive);
		}

		// straighten up lines first

		if (line < 0)
			line = 0;
		else
		{
			max = docMan.getMax();

			if (line >= max )
				line = max - 1;
		}

		// straighten up columns

		if (column < 0)
			column = 0;
		else
		{
			max = (docMan.getLine(line)).length();
			if (column > max)
				column = max;
		}

		// off page?

		if (shiftVert(line))
			paint = true;

		if (shiftHoriz(stringLength((docMan.getLine(line)).substring(0, column)) + EDGE))
			paint = true;

		if (paint)
			repaint();
		else
			release_cursor();
	}

	private boolean shiftVert( int line )
	{
		if (line < sy)
		{
			sy = line;
			if (sy < 0)
				sy = 0;
			vert.setValue(sy);
			return true;
		}

		if (line >= sy+ny)
		{
			sy = line - ny + 1;
			vert.setValue(sy);
			return true;
		}

		return false;
	}

	private boolean shiftHoriz( int x )
	{
		if (x <= sx)
		{
			sx = x - (dimension.width / 5);

			if (sx < 0)
				sx = 0;

			horiz.setValue(sx);
			return true;
		}

		if (x >= sx + dimension.width)
		{
			sx = x - dimension.width + (dimension.width / 5);
			horiz.setValue(sx);
			return true;
		}

		return false;
	}


	public void componentHidden(ComponentEvent e) {}
	public void componentShown(ComponentEvent e) {}
	public void componentMoved(ComponentEvent e) {}

	public void componentResized(ComponentEvent e)
	{
		redoControls(horiz.getValue(), vert.getValue(),true);
	}

	public void redoCanvas()
	{
		redoControls(horiz.getValue(), vert.getValue(),false);
	}

	private void redoControls(int h, int v, boolean clear)
	{
		int maxline, maxpix, bubble;

		Graphics g = getGraphics();

		if (g == null)
			return;

		updateFonts(g);

		if (g != null)
		{
			dimension = getSize();

			if (clear)
				clear_area(g);

			maxpix = 6000;
			bubble = dimension.width;

			if ( bubble > maxpix)
				maxpix = bubble;

			horiz.setValues(h, bubble, 0, maxpix);

			maxline = docMan.getMax();

			if ( maxline <= 0 )
				maxline = 10;

			bubble = ny = dimension.height / fontHeight;

			if (bubble > maxline)
				bubble = maxline;

			if (v > maxline)
				v = maxline;

			vert.setValues(v, bubble, 0, maxline);
			repaint();

			g.dispose();
		}
	}

	private void resizeLines()
	{
		if ((hline >= 0) && (hline < docMan.getMax()))
			opix = hpix = pix_at(hline,hcolumn);

		if ((heline >= 0) && (heline < docMan.getMax()))
			oepix = hepix = pix_at(heline,hecolumn);		
	}

	private int stringLength(String s)
	{
		if (fontMetrics == null)
			return -1;

		return fontMetrics.stringWidth(detabbed(s));
	}

	private String detabbed(String s)
	{
		if (s.indexOf('\t') < 0)
			return s;

		char c;
		String t = new String("");
		int j=0;
		int tabs;
		int max = s.length();

		for (int i=0; i<max; i++)
		{
			c = s.charAt(i);
			if (c == '\t')
			{
				tabs = tabSize - (j % tabSize);
				j += tabs;
				while (tabs-- > 0) t = t + ' ';
			}
			else
			{
				t = t + c;
				j++;
			}
		}

		return t;
	}

	public void getCursorPos(Graphics g, Rectangle r)
	{
		updateFonts(g);
		r.width = 2;
		r.height = fontHeight;
		r.y = (line - sy) * r.height;

		if (column > 0)
		{
			String s = (docMan.getLine(line)).substring(0, column);
			r.x = stringLength(s) + EDGE - sx;
		}
		else
			r.x = EDGE - sx;
	}

	public void adjustmentValueChanged(AdjustmentEvent e)
	{
		pause_cursor();
		if (e.getSource() == horiz)
		{
			sx = e.getValue();
			horiz.setValue(sx);
		}
		else
		if (e.getSource() == vert)
		{
			sy = e.getValue();
			vert.setValue(sy);
		}
		repaint();
	}

	public void linesChanged(int first, int last)
	{
		if ((first > sy+ny) || (last < sy))
			return;

		if (first == last)
		{
			Graphics g = getGraphics();
			updateFonts(g);
			drawLine(g, first);
			g.dispose();
			release_cursor();
		}
		else
			repaint();
	}

	public void update(Graphics g)
	{ 
		paint(g);
	}

	public void paint(Graphics g)
	{ 
		int i;
		Rectangle clip;
		Rectangle curs = new Rectangle();
		
		pause_cursor();

		g.setPaintMode();
		updateFonts(g);

		i = docMan.getMax();
		if (oldlines != i)
		{
			oldlines = i;
			redoControls(horiz.getValue(), vert.getValue(),true);
		}
		
		if (dimension == null)
			dimension = getSize();

		clip = g.getClipBounds();

		i = clip.y / fontHeight;
		
		if ( clip.height / fontHeight == 1)
			drawLine(g, sy + i);
		else
			for (i = 0; i < ny; i++)
				drawLine(g,sy+i);

		release_cursor();
	}
	
	private synchronized void drawLine(Graphics g, int i)
	{
		int x,y,m;
		int bx,ex;
		String s;
		Rectangle curs = null;
				
		g.setPaintMode();
		g.setColor(Color.black);

		x = EDGE-sx;
		y = (i - sy + 1) * fontHeight;
		m = fontHeight;// + fontDescent;

		if (hactive && (i > oline) && (i < oeline))
			g.fillRect(0,y-fontHeight,dimension.width,m);
		else
			g.clearRect(0,y-fontHeight,dimension.width,m);
		
		if (i >= docMan.getMax())
			return;
			
		s = detabbed(docMan.getLine(i));
		
		if (hactive && (i > oline) && (i < oeline))
		{
			g.setColor(Color.white);
			g.drawString(s, x, y-fontDescent);
			g.setColor(Color.black);
		}
		else
			g.drawString(s, x, y-fontDescent);
	
		if (hactive && ((i == oline) || (i == oeline)))
		{
			bx = 0;
			ex = 5000;
			if (i == oline)
			{
				bx = opix - sx;
			}
			if (i == oeline)
			{
				ex = oepix - sx;
				if (oline == oeline)
					ex -= bx;
			}
			g.setXORMode(Color.white);
			g.fillRect(bx,y-fontHeight,ex,m);
		}
	}
}
